/* 
    FileName: main.cpp
    Date: Nov 16, 2010
    
    Collision Detection Example 
*/

#include "./source/PlatformSpecific/inc/GOC.h"

void GOC::Initialize()
{
 /* 
  Initialize your game scene, and game variables here. 
 */
    
    // Initilize ball.
    PointGOC ballPosition_l = { GetWindowWidth() / 2,
                                GetWindowHeight() / 2,
                                0.0f};

    ball_m = new Sprite();
    
    ball_m->Set2DPosition(ballPosition_l,
                          24,
                          24);
    ball_m->GetTexture()->LoadTexture("./images/Ball.png");
    
    ball_m->SetVisible(true);
    
    
    // Initilize player one.
    PointGOC playerOnePosition_l = { 15,
                                     15,
                                     0.0f};
    playerOne_m = new Sprite();
    
    playerOne_m->Set2DPosition(playerOnePosition_l,
                               15,
                               200);
    playerOne_m->GetTexture()->LoadTexture("./images/Padle.png");
    
    playerOne_m->SetVisible(true);
    
    // Initilize player two.
    
    PointGOC playerTwoPosition_l = { GetWindowWidth() - 30,
                                     15,
                                     0.0f};
    playerTwo_m = new Sprite();
    
    playerTwo_m->Set2DPosition(playerTwoPosition_l, 
                               15,
                               200);
    playerTwo_m->GetTexture()->LoadTexture("./images/Padle.png");
    
    playerTwo_m->SetVisible(true);
    
    maxPlayerPosition_m = GetWindowHeight() - playerOne_m->GetHeight();
    
    maxBallX_m = GetWindowWidth() - ball_m->GetWidth();
    maxBallY_m = GetWindowHeight() - ball_m->GetHeight();
    
    ballSpeed_m = 10;
    ballAngle_m = 60;
    
    ballVelocityX_m = ballSpeed_m * cos(ballAngle_m);
    ballVelocityY_m = ballSpeed_m * sin(ballAngle_m);    
}

void GOC::Update(TimeGOC *timeGOC_p, InputEngine *input_p)
{
    /* 
  Update your game scenes, check for collisions, check for input
  and play sounds here.
 */
    
    // Move the ball a little bit each frame.
    ball_m->SetXCor(ball_m->GetXcor() + ballVelocityX_m );
    ball_m->SetYCor(ball_m->GetYcor() + ballVelocityY_m );
    
    // Top Screen Edge Has been reached.
    if(ball_m->GetYcor() <= 0)
    {
        ballVelocityY_m = -ballVelocityY_m;
        ball_m->SetYCor(0);
    }
    
    // Bottom Screen Edge Has been reached.
    if(ball_m->GetYcor() >= maxBallY_m)
    {
        ballVelocityY_m = -ballVelocityY_m;
        ball_m->SetYCor(maxBallY_m);
    }
    
    // Right Screen Edge Has been reached.
    if(ball_m->GetXcor() >= maxBallX_m)
    {
        // Player One Scored.
        ballVelocityX_m = -ballVelocityX_m;
        ball_m->SetXCor(maxBallX_m);  
    }
    
    // Left Screen Edge Has been reached.
    if(ball_m->GetXcor() <= 0)
    {
        // Player Two Scored.
        ballVelocityX_m = -ballVelocityX_m;
        ball_m->SetXCor(0); 
    }
    
    // Ball collied with PlayerOne.
    if(playerOne_m->HasCollidedRight(ball_m) == CollidedSide::RIGHT_COLLISION)
    {
        ballVelocityX_m = -ballVelocityX_m;
        ball_m->SetXCor(31);
    }
    
    // Ball collied with PlayerTwo.
    if(playerTwo_m->HasCollidedLeft(ball_m) == CollidedSide::LEFT_COLLISIOIN)
    {
        ballVelocityX_m = -ballVelocityX_m;
        ball_m->SetXCor(maxBallX_m - 31);
    }    
    
    if(input_p->YCordinate() < maxPlayerPosition_m)
    {
        playerTwo_m->SetYCor(input_p->YCordinate());
    }
    else
    {   
        playerTwo_m->SetYCor(maxPlayerPosition_m);
    }
    
    if(input_p->IsKeyOrButtonPressed(KeysAndButtons::UP_KEYBOARD) && playerOne_m->GetYcor() > 0)
    {
        playerOne_m->SetYCor(playerOne_m->GetYcor() - 20);
    }
    
    if(input_p->IsKeyOrButtonPressed(KeysAndButtons::DOWN_KEYBOARD) && playerOne_m->GetYcor() < maxPlayerPosition_m)
    {
        playerOne_m->SetYCor(playerOne_m->GetYcor() + 20);
    }
}

void GOC::IndepedentUpdate(TimeGOC* timeGOC_p, InputEngine* input_p)
{
    /* 
  Update your game scenes, check for collisions, check for input
  and play sounds here.
 */
}

void GOC::Destroy()
{
 /* 
  Clean up after your game here.
 */
    
    delete ball_m;
    
    delete playerOne_m;
    delete playerTwo_m;
}

#ifdef _WIN32
    // Define the windows main funciton.
    int WINAPI WinMain(HINSTANCE hInstance, 
                       HINSTANCE hPrevInstance, 
                       LPSTR lpCmdLine, 
                       int nCmdShow)
#else
    // Define the ubutnu main function.
    int main(int argc, char **argv)
#endif
    {
        #ifdef _WIN32
            // Start the GOC on windows.
            GOC::goc_i()->LoadGOC(hInstance);
        #else
            // Start the GOC on ubuntu.
            GOC::goc_i()->LoadGOC();
        #endif
   
        return 0;
    }                                

